package com.gydx.component;

import java.io.IOException;
import java.util.concurrent.CopyOnWriteArraySet;
import javax.websocket.OnClose;
import javax.websocket.OnError;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Component;

/**
 * ----------------------- 竞价详情 -----------------------
 *   1.点击开始竞价，建立链接
 *   2.依据点击时间判断现在的倒计时，持续发送倒计时
 *   3.到达休息时间倒计时时，传递休息信息，关闭报价按钮，显示第一轮报价
 *   4.休息倒计时结束，重新开始倒计时，打开关闭按钮，知道轮次结束
 *   5.传递关闭信息，更改对应单号的状态为已经结素，同时关闭这个socket链接
 */
@ServerEndpoint(value = "/client/BiddingDetail/{userId}")
@Component
public class WSBiddingDetail {

    private static Logger logger = LogManager.getLogger(WSBiddingDetail.class.getName());

    /**
     * 静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
     */
    private static int onlineCount = 0;

    /**
     * concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
     */
    private static CopyOnWriteArraySet<WSBiddingDetail> wsClientMap = new CopyOnWriteArraySet<>();

    /**
     * 与某个客户端的连接会话，需要通过它来给客户端发送数据
     */
    private Session session;

    /**
     * 连接建立成功调用的方法
     *
     * @param session 当前会话session
     */
    @OnOpen
    public void onOpen(@PathParam("userId") String userId, Session session) {

        this.session = session;
        wsClientMap.add(this);
        addOnlineCount();
        logger.info(session.getId() + "有新链接加入，当前链接数为：" + wsClientMap.size());
        new Thread(() -> {
            int x = 100;
            while (x > 0) {
                try {
                    sendMessage(String.format("当前的值为：%d", x));
                    Thread.sleep(1000);
                } catch (Exception e) {
                    logger.error("倒计时错误", e);
                }
                --x;
            }
        }).start();
    }

    /**
     * 连接关闭
     */
    @OnClose
    public void onClose() {
        wsClientMap.remove(this);
        subOnlineCount();
        logger.info("有一链接关闭，当前链接数为：" + wsClientMap.size());
    }

    /**
     * 收到客户端消息
     *
     * @param message 客户端发送过来的消息
     * @param session 当前会话session
     */
    @OnMessage
    public void onMessage(String message, Session session) throws IOException, InterruptedException {
        logger.info("来终端的警情消息:" + message);
    }

    /**
     * 发生错误
     */
    @OnError
    public void onError(Session session, Throwable error) {
        logger.info("wsClientMap发生错误!");
        error.printStackTrace();
    }

    /**
     * 给所有客户端群发消息
     *
     * @param message 消息内容
     */
    public void sendMsgToAll(String message) throws IOException {
        for (WSBiddingDetail item : wsClientMap) {
            item.session.getBasicRemote().sendText(message);
        }
        logger.info("成功群送一条消息:" + wsClientMap.size());
    }

    public void sendMessage(String message) throws IOException {
        this.session.getBasicRemote().sendText(message);
        logger.info("成功发送一条消息:" + message);
    }

    public static synchronized int getOnlineCount() {
        return WSBiddingDetail.onlineCount;
    }

    public static synchronized void addOnlineCount() {
        WSBiddingDetail.onlineCount++;
    }

    public static synchronized void subOnlineCount() {
        WSBiddingDetail.onlineCount--;
    }
}
